/*
 * Config.h
 *
 * Copyright Notice: see copyright.txt
 *
 * Date: 3/8/2012 (D/M/Y)
 * Author: Dmitrij Lioubartsev
 * Email: dmitrijl42@gmail.com
 *
 * This class reads the configuration file and stores its parameters.
 * For config file specifications of parameters and syntax, see readme.txt.
 */

#ifndef CONFIG_H_
#define CONFIG_H_

#include <string>
#include <vector>
using namespace std;

class Config {
private:
	string output;// = "default";
	bool xmol;// = false;
	bool car; // = false
	double r_min;// = 20;
	//substitution
	vector<pair <string, double> > ratios; //example of value: "Si, 0.4".
	vector<string> exclude; //forbidden patterns, example "Al-O-Al".
	//Force Field
	string ff_file; // = "default"
	double charge_tot; // = 0

	//THIS IS NOT IN THE FILE
	bool fixCharge; // = false;

	//for two below methods, see CifParser, since the code is basically copy-pasted
	//with some modification.
	bool skipWhiteSpace(ifstream& stream);
	bool getWord(ifstream& stream, string& word);

public:
	Config();
	virtual ~Config();

	bool readFile(const char * configFile = "input.cfg"); //read the given config file.
	void createConf(); //create a config file with current values

	string getOutput() { return output; }
	bool getxmol() { return xmol; }
	bool getcar() { return car; }
	double getR_min() { return r_min; }
	vector<pair <string, double> >* getSubRatios() { return &ratios;}
	vector<string>* getElims() { return &exclude;}
	string getFF_file() {return ff_file;}
	double getCharge_tot() {return charge_tot;}

	//Other
	bool getFixCharge() { return fixCharge;}
};

#endif

